//Bluetooth Devices Visualizer 1.0
//Jeremy Geppert
//2023


#include <Arduino.h>
#include "NimBLEDevice.h"
#include <Adafruit_SSD1306.h>
#include <FastLED.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

static boolean doScan = false;
static BLERemoteCharacteristic* pRemoteCharacteristic;
static BLEAdvertisedDevice* myDevice;
int scanIntervalSeconds = 10;
int maxDevicesSeen = 0;

/**
 * Scan for BLE servers and find the first one that advertises the service we are looking for.
 */
class MyAdvertisedDeviceCallbacks: public BLEAdvertisedDeviceCallbacks {
 /**
   * Called for each advertising BLE server.
   */
   
/*** Only a reference to the advertised device is passed now
  void onResult(BLEAdvertisedDevice advertisedDevice) { **/     
  void onResult(BLEAdvertisedDevice* advertisedDevice) {
    Serial.print("BLE Advertised Device found: ");
    Serial.println(advertisedDevice->toString().c_str());


  } // onResult
}; // MyAdvertisedDeviceCallbacks

int deviceCount = 0;
int page2deviceCount = 0;
int page3deviceCount = 0;

char devicenames;

#define LED_PIN     7
#define NUM_LEDS    256

CRGB leds[NUM_LEDS];

int buttonState = 0;
int modeNum = 1;

void setup() {
  Serial.begin(115200);

  FastLED.addLeds<WS2812, GPIO_NUM_26, GRB>(leds, NUM_LEDS);

    //begin display setup
    if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3D for 128x64
    Serial.println(F("SSD1306 allocation failed"));
    for(;;);
    }
    delay(2000);
    display.clearDisplay();

    display.setTextSize(3);
    display.setTextColor(WHITE);
    display.setCursor(0, 10);
    // Display static text
    display.println("BT VIZY V1.0");
    display.display();
    delay(2000);
    display.clearDisplay();
    display.setCursor(0, 10);
    //end display setup

  //Test the LED Matrix
  //Light them all up on low power RGB
  for (int i = 0; i <= NUM_LEDS-1; i++) {
    leds[i] = CRGB ( 5, 5, 5);
    FastLED.show();
    delay(10);
  }
  //Turn them all off
  for (int i = NUM_LEDS-1; i >= 0; i--) {
    leds[i] = CRGB ( 0, 0, 0);
    FastLED.show();
    delay(10);
  }
  
  Serial.println("Starting Arduino BLE Client application...");
  BLEDevice::init("");

  // Retrieve a Scanner and set the callback we want to use to be informed when we
  // have detected a new device.  Specify that we want active scanning and start the
  // scan to run for 5 seconds.
  
  //BLEScan* pBLEScan = BLEDevice::getScan();

  //pBLEScan->setAdvertisedDeviceCallbacks(new MyAdvertisedDeviceCallbacks());
  //pBLEScan->setInterval(1349);
  //pBLEScan->setWindow(449);
  //pBLEScan->setActiveScan(true);
  //pBLEScan->start(5, false);
  
} // End of setup.




void SignalStrengthMode()
{
  //Represents each device as a different color based on the RSSI value (signal strength)
  /*
  >= -70  STRONG Green?
  -70 to -85 GOOD Yellow?
  -86 to -100 FAIR Orange?
  < 100 POOR  Red?
  */

  BLEScan* pBLEScan = BLEDevice::getScan();
  pBLEScan->setInterval(1349);
  pBLEScan->setWindow(449);
  pBLEScan->setActiveScan(false);
  pBLEScan->start(scanIntervalSeconds, false);
    
  deviceCount = pBLEScan->getResults().getCount();

  int deviceRssi = 0;

  FastLED.clear();

  for (uint32_t i = 0; i < deviceCount; i++)
  {
    BLEAdvertisedDevice currentDevice = pBLEScan->getResults().getDevice(i);
    deviceRssi = abs(currentDevice.getRSSI()); //Remove the - so my thinking about this doesn't break my brain
   
    //By Color Mode
    if(deviceRssi <= 70){
      leds[i] = CRGB ( 0, 10, 0); //green
    }
    if(deviceRssi > 70 && deviceRssi <= 85){
      leds[i] = CRGB ( 10, 10, 0);//yellow
    }
    if(deviceRssi > 85 && deviceRssi <= 100){
      leds[i] = CRGB ( 10, 7, 0);//orange
    }
    if(deviceRssi > 100){
      leds[i] = CRGB ( 10, 0, 0);//red
    }
    

    //By Intensity
    /*
    if(deviceRssi <= 70){
      leds[i] = CRGB ( 0, 0, 30);//bright
    }
    if(deviceRssi > 70 && deviceRssi <= 85){
      leds[i] = CRGB ( 0, 0, 20);//not as bright
    }
    if(deviceRssi > 85 && deviceRssi <= 100){
      leds[i] = CRGB ( 0, 0, 10);//dim
    }
    if(deviceRssi > 100){
      leds[i] = CRGB ( 0, 0, 1);//dimmer
    }
    */
    FastLED.show();

  }

  //OLED Display
  display.clearDisplay();
  display.setCursor(0, 10);
  display.print("Devices " + String(deviceCount));
  display.display();

}

void NormalPagingBlueMode()
{

    FastLED.clear();

    BLEScan* pBLEScan = BLEDevice::getScan();
    pBLEScan->setInterval(1349);
    pBLEScan->setWindow(449);
    pBLEScan->setActiveScan(false);
    pBLEScan->start(scanIntervalSeconds, false);
    
    deviceCount = pBLEScan->getResults().getCount();

    
    //For testing, override device count variable 
    //deviceCount = 550;

    int MultipleOf64Count = deviceCount/64; 
    int FinalPageCount = deviceCount - (MultipleOf64Count*64);

    display.clearDisplay();
    display.setCursor(0, 10);
    
    for (int i = 0; i <= FinalPageCount-1; i++) {
      leds[i] = CRGB ( 0, 0, 5);
      FastLED.show();
    }
    
    display.print("Devices " + String(deviceCount) + ":P" + String(MultipleOf64Count));
    display.display();
}

void PagingSignalStrengthMode()
{

    FastLED.clear();

    BLEScan* pBLEScan = BLEDevice::getScan();
    pBLEScan->setInterval(1349);
    pBLEScan->setWindow(449);
    pBLEScan->setActiveScan(false);
    pBLEScan->start(scanIntervalSeconds, false);
    
    deviceCount = pBLEScan->getResults().getCount();

    
    //For testing, override device count variable 
    //deviceCount = 550;

    int MultipleOf64Count = deviceCount/256; 
    int FinalPageCount = deviceCount - (MultipleOf64Count*256);
    
    int deviceRssi = 0;

    display.clearDisplay();
    display.setCursor(0, 10);
    
    //To be able to handle the paging simply, this is showing the first 64 devices signal strengths
    for (int i = 0; i <= FinalPageCount-1; i++) {
      
      BLEAdvertisedDevice currentDevice = pBLEScan->getResults().getDevice(i);
      deviceRssi = abs(currentDevice.getRSSI()); //Remove the - so my thinking about this doesn't break my brain
   
      //By Color Mode
      if(deviceRssi <= 70){
        leds[i] = CRGB ( 0, 10, 0); //green
      }
      if(deviceRssi > 70 && deviceRssi <= 85){
        leds[i] = CRGB ( 10, 10, 0);//yellow
      }
      if(deviceRssi > 85 && deviceRssi <= 100){
        leds[i] = CRGB ( 10, 3, 0);//orange
      }
      if(deviceRssi > 100){
        leds[i] = CRGB ( 10, 0, 0);//red
      }
      FastLED.show();
    }
    
    display.print("Devices " + String(deviceCount) + ":P" + String(MultipleOf64Count));
    display.display();
}

void OriginalMode()
{
    BLEScan* pBLEScan = BLEDevice::getScan();
    pBLEScan->setInterval(1349);
    pBLEScan->setWindow(449);
    pBLEScan->setActiveScan(false);
    pBLEScan->start(scanIntervalSeconds, false);

    
    deviceCount = pBLEScan->getResults().getCount();

    //For testing, override device count variable 
    //deviceCount = 550;

  FastLED.clear();

  //Case for 0-64 devices
  if(deviceCount <= 64)
  {
    for (int i = 0; i <= deviceCount-1; i++) {
      leds[i] = CRGB ( 0, 0, 10);
      FastLED.show();
    }
    //Set the page indicator
    if(deviceCount > 7)
    {
      leds[7] = CRGB ( 10, 0, 10);
    }
    else
    {
      leds[7] = CRGB ( 10, 0, 0);
    }
    FastLED.show();
  }
  //Case for 65-128 devices
  else if(deviceCount <= 128)
  {
    page2deviceCount = deviceCount - 64;
    for (int i = 0; i <= page2deviceCount-1; i++) {
      leds[i] = CRGB ( 0, 0, 10);
      FastLED.show();
    }
    //Set the page indicator
    leds[7] = CRGB ( 10, 0, 0);
    if(deviceCount > 71)
    {
      leds[7] = CRGB ( 10, 0, 10);
      if(deviceCount > 80)
      {
        leds[8] = CRGB ( 10, 0, 10);
      }
      else
      {
        leds[8] = CRGB ( 10, 0, 0);
      }
    }
    else
    {
      leds[8] = CRGB ( 10, 0, 0);
    }
    FastLED.show();
  }
  else if(deviceCount <= 192)
  {
    page3deviceCount = deviceCount - 128;
    for (int i = 0; i <= page3deviceCount-1; i++) {
      leds[i] = CRGB ( 0, 0, 10);
      FastLED.show();
    }
    //Set the page indicator
    leds[7] = CRGB ( 10, 0, 0);
    leds[8] = CRGB ( 10, 0, 0);
    leds[23] = CRGB ( 10, 0, 0);
    FastLED.show();
  }
  else if(deviceCount <= 640)
  {
    //Each LED now represents 10 devices for 640 total possible devices
    int multipleDeviceCount = deviceCount / 10;
    for (int i = 0; i <= multipleDeviceCount-1; i++) {
      leds[i] = CRGB ( 0, 0, 10);
      FastLED.show();
    }

    //Set the the 4 corner leds as red to indicate 10x mode
    leds[7] = CRGB ( 10, 0, 0);
    leds[56] = CRGB ( 10, 0, 0);
    leds[63] = CRGB ( 10, 0, 0);
    leds[0] = CRGB ( 10, 0, 0);
    FastLED.show();
  }
  else
  {
    //Set the the 4 corner leds as red to indicate more than 640 and the end of display fun
    leds[7] = CRGB ( 0, 0, 10);
    leds[56] = CRGB ( 0, 0, 10);
    leds[63] = CRGB ( 0, 0, 10);
    leds[0] = CRGB ( 0, 0, 10);
    FastLED.show();
  }

//Serial and OLED display output
    Serial.print("Device Count: ");
    Serial.println(deviceCount);

    display.clearDisplay();
    display.setCursor(0, 10);

    if(deviceCount > 640){
      display.print("MaxMode " + String(deviceCount));
    }
    else if(deviceCount > 192){
      display.print("10xMode" + String(deviceCount));
    }
    else{
      display.print("Devices " + String(deviceCount));
    }
    display.display(); 
}

void BinaryDisplayMode()
{

//BINARY DISPLAY MODE
/*
      leds[0] = CRGB ( 0, 0, 10);  //1
      leds[15] = CRGB ( 0, 0, 10); //2
      leds[16] = CRGB ( 0, 0, 10); //4
      leds[31] = CRGB ( 0, 0, 10); //8
      leds[32] = CRGB ( 0, 0, 10); //16
      leds[47] = CRGB ( 0, 0, 10); //32
      leds[48] = CRGB ( 0, 0, 10); //64
      leds[63] = CRGB ( 0, 0, 10); //128

      FastLED.show();
*/

}




// This is the Arduino main loop function.
void loop() {
    
  switch (modeNum) {
  case 1:
    SignalStrengthMode();
    break;
  case 2:
    NormalPagingBlueMode();
    break;
  default:
    NormalPagingBlueMode();
    break;
  }


} 





